%PXMREAD Read a PXM (pbm, pgm, ppm) file from disk.
%
%  [R,G,B,pxmformat] = PXMREAD('filename') reads the PXM file 
%  'filename' and returns the RGB image in the matrices R,G,B,
%  and the format of the input file.  For the pbm (bitmap) format,
%  pxmformat == 'b',  for  the pgm (gray) format, pxmformat == 'g'
%  and for the pgm (color) format, pxmformat == 'p'. R=G=B when the
%  input file is in either the pbm (bitmap) or pgm (gray) format.
%  All values in R,G,B will be normalized to [0,1].  
%
%  [R,G,B] = PXMREAD('filename') reads the PXM file 'filename' 
%  and returns the RGB image in the matrices R,G,B. R=G=B when 
%  the input file is in the pbm (bitmap) or the pgm (gray) format.  
%  All values on R,G,B will be normalized to [0,1].  
%
%  [I,pxmformat] = PXMREAD('filename') reads the pbm or ppm image
%  named 'filename' and returns the image in the matrix I,
%  and the format of the input file.  For the pbm (bitmap) format,
%  pxmformat == 'b',  for the pgm (gray) format, pxmformat == 'g'.
%  When the input file is of the ppm format,  the function will 
%  exit with error message.  So, if the file format is unknown,
%  this usage is not recommended.  "I" will be normalized to [0,1].
%
%  I = PXMREAD('filename') reads the the pbm or the ppm format file 
%  'filename' then returns the image in the matrix "I",  When the 
%  input file is of the ppm format,  the function will exit with 
%  an error message.  So, if the file format is unknown, this usage 
%  is not recommended.  "I" will be normalized to [0,1].
%
%  See also  PXMWRITE.

%  Written by Hui Cheng  				Jan 9, 1996 
%  email: hui@ecn.purdue.edu

function [R,G,B,pxmformat] = pxmread(filename)

if (nargin~=1)
	error('Usage: [R,G,B,pxmformat] = pxmread(filename)') ;
end ;

if (isstr(filename)~=1)
	error('Requires a string filename as an argument.') ;
end ;

fid = fopen(filename,'r') ;
if (fid==-1)
	error(['Error opening ', filename, ' for input.']) ;
end

%  read header
str = fscanf(fid,'%s',1) ;
pxm = '0' ;
if(str==[])
	pxm = '0' ; 
elseif((str(1)=='P')&((str(2)=='4')|(str(2)=='5')|(str(2)=='6')))
	pxm = str(2) ;
end

max_iterms = 3 ;   % height, width and maxgray
if(pxm=='4')
	max_iterms = 2 ;  % height and width
end

num_iterms = 1 ;
while num_iterms <= max_iterms
	str  = fscanf(fid,'%s',1) ;
	if (prod(size(str)) > 70)|(str==[])
		pxm='0' ;
		break ;
	end

	if (str(1) == '#')
		chtemp = '1' ;
		while(~ (real(chtemp)==10))
		 	chtemp = fscanf(fid,'%c',1) ;
		 end
	
	else
		commandstr = sprintf('iterms%d=str ;',num_iterms) ;
		eval(commandstr) ;
		num_iterms = num_iterms+1 ;
	end
end

nC = str2num(iterms1) ;
nR = str2num(iterms2) ;
if (pxm=='4')
	maxgray = 255 ;
else
	maxgray = str2num(iterms3) ;
end

if (nC<=0)|(nR<=0)
	error('Height or width cannot be negative.') ;
end ;

if (maxgray<=0)|(maxgray>255)
	error('maxgray should be between 256 and 0.') ;
end ;

if (pxm~='0')
	chtemp = fscanf(fid,'%c',1) ;

	if (pxm=='4')
		iCount = round(nC / 8 + 0.5) * nR ;
		[temp,Count] = fread(fid,iCount, 'uchar') ;
		if (iCount~=Count)
			error('Not enough data.') ;
		end ;
		
		G = zeros(iCount/nR*8, nR) ;
		mask = [128,64,32,16,8,4,2,1] ;
		for (i=1:8)
			Index = find(temp>=mask(i)) ;
			temp(Index) = temp(Index)-mask(i) ;
			G(Index*8-8+i) = ones(size(Index)) ;
		end ;
		G = G' ;
		R = zeros(nR,nC) ;
		R = G(:,1:nC) ;
		
		if (find(R~=1 & R~=0)~=[])
			error('Invalid data.  Data should be 0 or 1.');
		end
		
	elseif (pxm=='5')
		iCount = nC*nR ;
		[temp,Count] = fread(fid,[nC,nR],'uchar') ;
		if (iCount~=Count)
			error('Not enough data.') ;
		end ;
		R = (temp/maxgray)' ;
		
		if (find(R<0 | R>1)~=[])
			error('Invalid data. Data should be between 0 and 1.');
		end
		
	elseif (pxm=='6')
		if (nargout < 3)
		error('The input file is .ppm file, needs at least 3 output matrices.');
		end
		
		iCount = nC*nR*3 ;
		[temp,Count] = fread(fid,[nC*3,nR], 'uchar') ;
		if (iCount~=Count)
			error('Not enough data.') ;
		end ;
		
		temp = temp / maxgray ;
		Ind = 1:1:(nC*nR) ;
		R = zeros(nC,nR) ;
		G = zeros(nC,nR) ;
		B = zeros(nC,nR) ;
		R(Ind) = (temp(3*Ind-2)) ;
		G(Ind) = (temp(3*Ind-1)) ;
		B(Ind) = (temp(3*Ind)) ;
		R = R' ;
		G = G' ;
		B = B' ;
		if (find(R<0 | R>1)~=[])
			error('Invalid data. Data should be between 0 and 1.');
		end
		if (find(G<0 | G>1)~=[])
			error('Invalid data. Data should be between 0 and 1.');
		end
		if (find(B<0 | B>1)~=[])
			error('Invalid data. Data should be between 0 and 1.');
		end
		
	else
		error('Invalid type of input file.') ;
	end

end
fclose(fid) ;

if (nargout == 1)

elseif (nargout == 2)
	if (pxm == '4')
		G = 'b' ;
	elseif (pxm == '5')
		G = 'g' ;
	else
		error('Invalid type of input file or not enough number of outputs') ;
	end

elseif (nargout == 3)
	if (pxm == '4' | pxm == '5')
		G = R ;
		B = R ;
	end
elseif (nargout == 4)
	if (pxm == '4' | pxm == '5')
		G = R ;
		B = R ;
	end
	
	if (pxm == '4')
		pxmformat = 'b' ;
	elseif (pxm == '5')
		pxmformat = 'g' ;
	elseif (pxm == '6')
		pxmformat = 'p' ;
	else
		error('Invalid type of input file.') ;
	end
end
